#include <SoftwareSerial.h>
#include "FastLED.h"

SoftwareSerial BT(12,13);
int mode = 0;

#define NUM_LEDS 43
#define DATA_PIN 8
#define delay_spectrum 0
#define ambiserial 115200
#define blueserial 9600
CRGB leds[NUM_LEDS];

String RGB_string = "";
boolean RGB_Completed = false;
uint8_t prefix[] = {'A', 'd', 'a'}, hi, lo, chk, i;

const int buttonPin = 2;
unsigned long lastDebounceTime = 0;
unsigned long debounceDelay = 500000;
int counter_time = 0;

ISR(TIMER1_COMPA_vect) {
  if (counter_time > 20000) {
    counter_time = 0;
  }
  counter_time++;
}

ISR(INT0_vect) {
  if((long)(micros() - lastDebounceTime) >= debounceDelay) {
    mode++;
    lastDebounceTime = micros();
  }
}

void setup() {
  FastLED.addLeds<NEOPIXEL, DATA_PIN>(leds, NUM_LEDS);
  Serial.begin(ambiserial);
  BT.begin(blueserial);
  RGB_string.reserve(30);
  Serial.print("Ada\n");
  LEDS.showColor(CRGB(0, 0, 0));

  EICRA |= (1 << ISC00) | (1 << ISC01);
  EIMSK |= (1 << INT0);

  TCCR1A = 0;
  TCCR1B = 0;
  TCNT1 = 0;
  OCR1A = 2000;
  TCCR1B |= (1 << WGM12);
  TCCR1B |= (1 << CS12);
}

void loop() {
  if(mode % 3 == 0){
    TIMSK1 |= (1 << OCIE1A);
    spectrum();
    TIMSK1 &= ~(1 << OCIE1A);
  }
  if(mode % 3 == 1){
    bluetooth();
  }
  if(mode % 3 == 2){
    ambi();
  }
}

void Light_RGB_LED(){
  int SP1 = RGB_string.indexOf('.');
  int SP2 = RGB_string.indexOf('.', SP1+1);
  int SP3 = RGB_string.indexOf('.', SP2+1);
  String R = RGB_string.substring(0, SP1);
  String G = RGB_string.substring(SP1+1, SP2);
  String B = RGB_string.substring(SP2+1, SP3);
  LEDS.showColor(CRGB(R.toInt(), G.toInt(), B.toInt()));
}

void spectrum(){
  int r = 120, g = 0, b = 255;
  LEDS.showColor(CRGB(r, g, b));
  int aux = counter_time;
  while(aux == counter_time) {
    Serial.println(counter_time);
  }
  for(int i = 1; i <= 120; i++) {
    if (mode % 3 != 0) return;
    r--;
    LEDS.showColor(CRGB(r, g, b));
    aux = counter_time;
    while(aux == counter_time) {
      Serial.println(counter_time);
    }
  }
  for(int i = 1; i <= 255; i++) {
    if (mode % 3 != 0) return;
    g++;
    LEDS.showColor(CRGB(r, g, b));
    aux = counter_time;
    while(aux == counter_time) {
      Serial.println(counter_time);
    }
  }
  for(int i = 1; i <= 255; i++) {
    if (mode % 3 != 0) return;
    g--;
    LEDS.showColor(CRGB(r, g, b));
    aux = counter_time;
    while(aux == counter_time) {
      Serial.println(counter_time);
    }
  }
  for(int i = 1; i <= 120; i++) {
    if (mode % 3 != 0) return;
    r++;
    LEDS.showColor(CRGB(r, g, b));
    aux = counter_time;
    while(aux == counter_time) {
      Serial.println(counter_time);
    }
  }

}

void bluetooth(){
  while(BT.available()){
      char ReadChar = (char)BT.read();
      if(ReadChar == ')'){
        RGB_Completed = true;
      }else{
        RGB_string += ReadChar;
      }
    }
    if(RGB_Completed){
        Light_RGB_LED();  
      }
      RGB_string = "";
      RGB_Completed = false;
    }
}

void ambi(){
  if (Serial.available()){
    for(i = 0; i < sizeof prefix; ++i) {
      waitLoop: while (!Serial.available()) ;
      if(prefix[i] == Serial.read()) continue;
      i = 0;
      goto waitLoop;
    }
    while (!Serial.available()) ;;
    hi=Serial.read();
    while (!Serial.available()) ;;
    lo=Serial.read();
    while (!Serial.available()) ;;
    
    chk=Serial.read();

    if (chk != (hi ^ lo ^ 0x55)) {
      i=0;
      goto waitLoop;
    }

    memset(leds, 0, NUM_LEDS * sizeof(struct CRGB));

    for (uint8_t i = 0; i < NUM_LEDS; i++) {
      byte r, g, b;    
      while(!Serial.available());
      r = Serial.read();
      while(!Serial.available());
      g = Serial.read();
      while(!Serial.available());
      b = Serial.read();
      leds[i].r = r;
      leds[i].g = g;
      leds[i].b = b;
    }
    FastLED.show();
  }
}
  
